/*
 * CImagenexSidescanSonar.h
 *
 *  Created on: Mar 2, 2011
 *      Author: Joel Vidal
 */

#ifndef CIMAGENEXSIDESCANSONAR_H_
#define CIMAGENEXSIDESCANSONAR_H_

#include "../core/CComponent.h"
#include "../datamanip/CDataManip.h"


#include "../core/io/IIODevice.h"
#include "../core/threading/IThread.h"

//device_
#include "../core/io/dev/CTcpSocket.h"
#include "../datamanip/specific/CImagenexSidescanSonarDataManip.h"

#include "opencv/cv.h"
#include "opencv/highgui.h"

#include "Poco/DateTime.h"

class CImagenexSidescanSonar : public CComponent {
public:

	struct Config {
		int deviceTimeout ;
		std::string deviceIP ;
		int devicePort ;
		int range ;
		int frequency ;
		int dataGain ;
		int balanceGain ;
		bool imageCreate ;
		int imageWidthPx ;
		int imageHeightPx ;
		int imagePing ;
		int imageRefresh ;
	};

	class ConfigDataManip : public CDataManip< Config > {
	public:
		ConfigDataManip( const std::string& rootTagName ):
			CDataManip< Config >( rootTagName )
		{
			addItem( "device_timeout", &Config::deviceTimeout ) ;
			addItem( "device_ip", &Config::deviceIP ) ;
			addItem( "device_port", &Config::devicePort ) ;
			addItem( "range", &Config::range ) ;
			addItem( "frequency", &Config::frequency ) ;
			addItem( "data_gain", &Config::dataGain ) ;
			addItem( "balance_gain", &Config::balanceGain ) ;
			addItem( "image_create", &Config::imageCreate ) ;
			addItem( "image_width_px", &Config::imageWidthPx ) ;
			addItem( "image_height_px", &Config::imageHeightPx ) ;
			addItem( "image_ping", &Config::imagePing ) ;
			addItem( "image_refresh", &Config::imageRefresh ) ;
		}
	} ;

	explicit
	CImagenexSidescanSonar( const std::string& componentName ) ;

	~CImagenexSidescanSonar() {} ;


	void
	initialize()
		throw ( std::runtime_error ) ;


	std::string
	getConfiguration() const
		throw ( std::runtime_error ) ;


	void
	setConfiguration( const std::string& configuration )
		throw ( std::runtime_error ) ;



private:

	boost::shared_ptr< IThread > thread_ ;
	boost::shared_ptr< IIODevice > device_ ;
	CImagenexSidescanSonarDataManip imagenexSidescanSonarDataManip_ ;
	ConfigDataManip configDataManip_ ;

	struct SImage {
		bool create;

		cv::Mat imageView ;
		char * circularBuffer ;
		int bufferSize ;
		int bufferPos ;

		int width ;
		int height ;

		int iniRefreshTime ;
		int refreshTime ;

		int iniPingTime ;
		int pingTime ;
		bool firstPingEnd ;
	};

	SImage image_ ;

	void
	run() ;

	/*
	 *  Send Switch Data Command
	 */
	void
	sendSDC( unsigned char packetNumber );

	/*
	 *  Recieve Interface Retrun Data
	 */
	void
	recieveIRD( unsigned char packetNumber ) ;

	void
	imageInitializeView() ;

	void
	imageStopView() ;

	void
	imageAddChannel( const IIODevice::DataBuffer & channel ,
				     unsigned char packetNumber ) ;

	void
	imageDraw() ;

	void
	imageScaleChannel( const IIODevice::DataBuffer & channelIn ,
					   IIODevice::DataBuffer & channelOut ) ;

	void
	imageAddInfo() ;

};

#endif /* CIMAGENEXSIDESCANSONAR_H_ */
